/* minimal program to open & clear a window */
#include <cmath>
#include <iostream>
#include <GL/glew.h>
#include <GL/freeglut.h>
#include <GL/freeglut_ext.h>
#include "moglclass.h"
#include "shaders.h"
#include "vertexBufferObject.h"

using namespace moglclass;

#define NUMSHADERS 2
Shader shShaders[NUMSHADERS];
Program spMain;

GLuint vao;
VertexBufferObject vbo;


////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

// set color triangle
void setData()
{
	// VBO
	vbo.addData(&glm::vec4(-0.75f, -0.75f, 0.0f, 1.0f), sizeof(glm::vec4)); // vertex1 position
	vbo.addData(&glm::vec4(1.0f, 0.0f, 0.0f, 1.0f), sizeof(glm::vec4));		// vertex1 color
	vbo.addData(&glm::vec4(0.75f, -0.75f, 0.0f, 1.0f), sizeof(glm::vec4));	// vertex2 position
	vbo.addData(&glm::vec4(0.0f, 1.0f, 0.0f, 1.0f), sizeof(glm::vec4));		// vertex2 color
	vbo.addData(&glm::vec4(0.75f, 0.75f, 0.0f, 1.0f), sizeof(glm::vec4));	// vertex3 position
	vbo.addData(&glm::vec4(0.0f, 0.0f, 1.0f, 1.0f), sizeof(glm::vec4));		// vertex3 color

	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	// VAO
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	// vertex attrib
	int iDataStride = 2 * sizeof(glm::vec4); // vertex & color
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec4);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 4, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
}

void init( void )
{
    // Load shaders and use the resulting shader program
	spMain.createProgram();
	shShaders[0].loadShader("simple2.vs", GL_VERTEX_SHADER);
	shShaders[1].loadShader("simple2.fs", GL_FRAGMENT_SHADER);
	spMain.attachShaderToProgram(&shShaders[0]);
	spMain.attachShaderToProgram(&shShaders[1]);
	spMain.linkProgram();
	spMain.useProgram();

	setData();

	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
}

void display( void )
{
	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
	glClear( GL_COLOR_BUFFER_BIT ); // clear the window

	glDrawArrays(GL_TRIANGLES, 0, 3);	// draw triangle

	glFlush();
}

void keyboard( unsigned char key, int x, int y )
{
	switch ( key ) {
		case 27:
			exit( EXIT_SUCCESS );
			break;
	}
}

void cleanup()
{		 
	spMain.deleteProgram();
}

int main( int argc, char **argv )
{ 
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_RGBA );
	glutInitWindowSize( 512, 512 );
	glutCreateWindow( "Your First OpenGL" );

	glewInit();
	init();
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutMainLoop();

	cleanup();
	return 0;
}